﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirFighter
{
    internal class MapsCollection
    {
        readonly Dictionary<string, MapWithSetCarsGeneric<IDrawingObject, AbstractMap>> _mapStorages;

        public List<string> Keys => _mapStorages.Keys.ToList();

        private readonly int _pictureWidth;
        private readonly int _pictureHeight;

        private char separatorDict = '|';
        private char separatorData = ';';

        public MapsCollection(int pictureWidth, int pictureHeight)
        {
            _mapStorages = new Dictionary<string, MapWithSetCarsGeneric<IDrawingObject, AbstractMap>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        /// <summary>
        /// Добавление карты
        /// </summary>
        /// <param name="name">Название карты</param>
        /// <param name="map">Карта</param>
        public void AddMap(string name, AbstractMap map)
        {
            // TODO Прописать логику для добавления
            bool check = _mapStorages.TryGetValue(name, out var value);
            if (check) return;

            _mapStorages[name] = new(_pictureWidth, _pictureHeight, map);
        }

        public void DelMap(string name)
        {
            // TODO Прописать логику для удаления
            _mapStorages.Remove(name);
        }

        public void SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            using (StreamWriter fs = new(filename))
            {
                fs.Write($"MapsCollection{Environment.NewLine}");
                foreach (var storage in _mapStorages)
                {

                    fs.Write($"{storage.Key}{separatorDict}{storage.Value.GetData(separatorDict, separatorData)}{Environment.NewLine}");
                }
            }
        }

        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new FileNotFoundException("Файл не найден");
            }
            using (StreamReader fs = new(filename))
            {
                string current = fs.ReadLine();
                if (!current.Contains("MapsCollection"))
                {
                    //если нет такой записи, то это не те данные
                    throw new FileFormatException("Не правильный формат данных");
                }

                _mapStorages.Clear();

                while ((current = fs.ReadLine()) != null)
                {
                    var elem = current.Split(separatorDict);
                    AbstractMap map = null;
                    switch (elem[1])
                    {
                        case "SimpleMap":
                            map = new SimpleMap();
                            break;
                        case "MyMap":
                            map = new MyMap();
                            break;
                    }
                    _mapStorages.Add(elem[0], new MapWithSetCarsGeneric<IDrawingObject, AbstractMap>(_pictureWidth, _pictureHeight, map));
                    _mapStorages[elem[0]].LoadData(elem[2].Split(separatorData, StringSplitOptions.RemoveEmptyEntries));
                }
            }
        }


        public MapWithSetCarsGeneric<IDrawingObject, AbstractMap> this[string ind]
        {
            get
            {
                // TODO Продумать логику получения объекта
                bool check = _mapStorages.TryGetValue(ind, out var mapWithSet);
                return mapWithSet;
            }
        }
    }

}
